@@ -23,6 +23,20 @@
  OTHER DEALINGS IN THE SOFTWARE.
 """
 
-from errors import *
-from client import Site, __ver__
-import ex
+from mwclient.errors import *  # noqa: F401, F403 # pylint: disable=unused-import
+from mwclient.client import Site, __version__  # noqa: F401 # pylint: disable=unused-import
+import logging
+import warnings
+
+# Show DeprecationWarning
+warnings.simplefilter('always', DeprecationWarning)
+
+# Logging: Add a null handler to avoid "No handler found" warnings.
+try:
+    from logging import NullHandler
+except ImportError:
+    class NullHandler(logging.Handler):
+        def emit(self, record):
+            pass
+
+logging.getLogger(__name__).addHandler(NullHandler())
