@@ -17,17 +17,19 @@ from requests_oauthlib import OAuth1
 import mwclient.errors as errors
 import mwclient.listing as listing
 from mwclient.sleep import Sleepers
-from mwclient.util import parse_timestamp
+from mwclient.util import parse_timestamp, read_in_chunks
 
 try:
     import gzip
 except ImportError:
     gzip = None
 
-__ver__ = '0.8.7'
+__version__ = '0.10.0'
 
 log = logging.getLogger(__name__)
 
+USER_AGENT = 'mwclient/{} ({})'.format(__version__, 'https://github.com/mwclient/mwclient')
+
 
 class Site(object):
     """A MediaWiki site identified by its hostname.
@@ -53,7 +55,7 @@ class Site(object):
                  max_retries=25, wait_callback=lambda *x: None, clients_useragent=None,
                  max_lag=3, compress=True, force_login=True, do_init=True, httpauth=None,
                  reqs=None, consumer_token=None, consumer_secret=None, access_token=None,
-                 access_secret=None, client_certificate=None, custom_headers=None):
+                 access_secret=None, client_certificate=None, custom_headers=None, scheme='https'):
         # Setup member variables
         self.host = host
         self.path = path
@@ -63,6 +65,9 @@ class Site(object):
         self.max_lag = text_type(max_lag)
         self.force_login = force_login
         self.requests = reqs or {}
+        self.scheme = scheme
+        if 'timeout' not in self.requests:
+            self.requests['timeout'] = 30  # seconds
 
         if consumer_token is not None:
             auth = OAuth1(consumer_token, consumer_secret, access_token, access_secret)
@@ -93,14 +98,13 @@ class Site(object):
             if client_certificate:
                 self.connection.cert = client_certificate
 
-            prefix = '{} - '.format(clients_useragent) if clients_useragent else ''
-            self.connection.headers['User-Agent'] = (
-                '{prefix}MwClient/{ver} ({url})'.format(
-                    prefix=prefix,
-                    ver=__ver__,
-                    url='https://github.com/mwclient/mwclient'
-                )
-            )
+            # Set User-Agent header field
+            if clients_useragent:
+                ua = clients_useragent + ' ' + USER_AGENT
+            else:
+                ua = USER_AGENT
+            self.connection.headers['User-Agent'] = ua
+
             if custom_headers:
                 self.connection.headers.update(custom_headers)
         else:
@@ -119,12 +123,15 @@ class Site(object):
         # Initialization status
         self.initialized = False
 
+        # Upload chunk size in bytes
+        self.chunk_size = 1048576
+
         if do_init:
             try:
                 self.site_init()
             except errors.APIError as e:
                 if e.args[0] == 'mwoauth-invalid-authorization':
-                    raise errors.OAuthAuthorizationError(e.code, e.info)
+                    raise errors.OAuthAuthorizationError(self, e.code, e.info)
 
                 # Private wiki, do init after login
                 if e.args[0] not in {u'unknown_action', u'readapidenied'}:
@@ -308,8 +315,10 @@ class Site(object):
                 return False
 
             # cope with https://phabricator.wikimedia.org/T106066
-            if (info['error'].get('code') == u'mwoauth-invalid-authorization' and
-               'Nonce already used' in info['error'].get('info')):
+            if (
+                info['error'].get('code') == u'mwoauth-invalid-authorization'
+                and 'Nonce already used' in info['error'].get('info')
+            ):
                 log.warning('retrying due to nonce error https://phabricator.wikimedia.org/T106066')
                 sleeper.sleep()
                 return False
@@ -349,6 +358,7 @@ class Site(object):
             data (dict): Post data
             files (dict): Files to upload
             retry_on_error (bool): Retry on connection error
+            http_method (str): The HTTP method, defaults to 'POST'
 
         Returns:
             The raw text response.
@@ -358,9 +368,14 @@ class Site(object):
             headers['Accept-Encoding'] = 'gzip'
         sleeper = self.sleepers.make((script, data))
 
-        scheme = 'https'
+        scheme = self.scheme
         host = self.host
         if isinstance(host, (list, tuple)):
+            warnings.warn(
+                'Specifying host as a tuple is deprecated as of mwclient 0.10.0. '
+                + 'Please use the new scheme argument instead.',
+                DeprecationWarning
+            )
             scheme, host = host
 
         url = '{scheme}://{host}{path}{script}{ext}'.format(scheme=scheme, host=host,
@@ -369,12 +384,16 @@ class Site(object):
 
         while True:
             try:
+                args = {'files': files, 'headers': headers}
+                for k, v in self.requests.items():
+                    args[k] = v
                 if http_method == 'GET':
-                    stream = self.connection.get(url, params=data, files=files,
-                                                 headers=headers, **self.requests)
+                    args['params'] = data
                 else:
-                    stream = self.connection.post(url, data=data, files=files,
-                                                  headers=headers, **self.requests)
+                    args['data'] = data
+
+                stream = self.connection.request(http_method, url, **args)
+
                 if stream.headers.get('x-database-lag'):
                     wait_time = int(stream.headers.get('retry-after'))
                     log.warning('Database lag exceeds max lag. '
@@ -486,7 +505,29 @@ class Site(object):
         return info
 
     def login(self, username=None, password=None, cookies=None, domain=None):
-        """Login to the wiki."""
+        """
+        Login to the wiki using a username and password. The method returns
+        nothing if the login was successful, but raises and error if it was not.
+
+        Args:
+            username (str): MediaWiki username
+            password (str): MediaWiki password
+            cookies (dict): Custom cookies to include with the log-in request.
+            domain (str): Sends domain name for authentication; used by some
+                MediaWiki plug-ins like the 'LDAP Authentication' extension.
+
+        Raises:
+            LoginError (mwclient.errors.LoginError): Login failed, the reason can be
+                obtained from e.code and e.info (where e is the exception object) and
+                will be one of the API:Login errors. The most common error code is
+                "Failed", indicating a wrong username or password.
+
+            MaximumRetriesExceeded: API call to log in failed and was retried until all
+                retries were exhausted. This will not occur if the credentials are merely
+                incorrect. See MaximumRetriesExceeded for possible reasons.
+
+            APIError: An API error occurred. Rare, usually indicates an internal server error.
+        """
 
         if username and password:
             self.credentials = (username, password, domain)
@@ -520,7 +561,8 @@ class Site(object):
                 elif login['login']['result'] == 'Throttled':
                     sleeper.sleep(int(login['login'].get('wait', 5)))
                 else:
-                    raise errors.LoginError(self, login['login'])
+                    raise errors.LoginError(self, login['login']['result'],
+                                            login['login']['reason'])
 
         self.site_init()
 
@@ -612,20 +654,34 @@ class Site(object):
         if not image.can('upload'):
             raise errors.InsufficientPermission(filename)
 
-        predata = {}
-
         if comment is None:
-            predata['comment'] = description
+            comment = description
+            text = None
         else:
-            predata['comment'] = comment
-            predata['text'] = description
+            comment = comment
+            text = description
+
+        if file is not None:
+            if not hasattr(file, 'read'):
+                file = open(file, 'rb')
+
+            content_size = file.seek(0, 2)
+            file.seek(0)
+
+            if self.version[:2] >= (1, 20) and content_size > self.chunk_size:
+                return self.chunk_upload(file, filename, ignore, comment, text)
+
+        predata = {
+            'action': 'upload',
+            'format': 'json',
+            'filename': filename,
+            'comment': comment,
+            'text': text,
+            'token': image.get_token('edit'),
+        }
 
         if ignore:
             predata['ignorewarnings'] = 'true'
-        predata['token'] = image.get_token('edit')
-        predata['action'] = 'upload'
-        predata['format'] = 'json'
-        predata['filename'] = filename
         if url:
             predata['url'] = url
 
@@ -645,11 +701,7 @@ class Site(object):
             # Since the filename in Content-Disposition is not interpreted,
             # we can send some ascii-only dummy name rather than the real
             # filename, which might contain non-ascii.
-            file = ('fake-filename', file)
-            # End of workaround
-            # ----------------------------------------------------------------
-
-            files = {'file': file}
+            files = {'file': ('fake-filename', file)}
 
         sleeper = self.sleepers.make()
         while True:
@@ -658,7 +710,72 @@ class Site(object):
             if not info:
                 info = {}
             if self.handle_api_result(info, kwargs=predata, sleeper=sleeper):
-                return info.get('upload', {})
+                response = info.get('upload', {})
+                break
+        if file is not None:
+            file.close()
+        return response
+
+    def chunk_upload(self, file, filename, ignorewarnings, comment, text):
+        """Upload a file to the site in chunks.
+
+        This method is called by `Site.upload` if you are connecting to a newer
+        MediaWiki installation, so it's normally not necessary to call this
+        method directly.
+
+        Args:
+            file (file-like object): File object or stream to upload.
+            params (dict): Dict containing upload parameters.
+        """
+        image = self.Images[filename]
+
+        content_size = file.seek(0, 2)
+        file.seek(0)
+
+        params = {
+            'action': 'upload',
+            'format': 'json',
+            'stash': 1,
+            'offset': 0,
+            'filename': filename,
+            'filesize': content_size,
+            'token': image.get_token('edit'),
+        }
+        if ignorewarnings:
+            params['ignorewarnings'] = 'true'
+
+        sleeper = self.sleepers.make()
+        offset = 0
+        for chunk in read_in_chunks(file, self.chunk_size):
+            while True:
+                data = self.raw_call('api', params, files={'chunk': chunk})
+                info = json.loads(data)
+                if self.handle_api_result(info, kwargs=params, sleeper=sleeper):
+                    response = info.get('upload', {})
+                    break
+
+            offset += chunk.tell()
+            chunk.close()
+            log.debug('%s: Uploaded %d of %d bytes', filename, offset, content_size)
+            params['filekey'] = response['filekey']
+            if response['result'] == 'Continue':
+                params['offset'] = response['offset']
+            elif response['result'] == 'Success':
+                file.close()
+                break
+            else:
+                # Some kind or error or warning occured. In any case, we do not
+                # get the parameters we need to continue, so we should return
+                # the response now.
+                file.close()
+                return response
+
+        del params['action']
+        del params['stash']
+        del params['offset']
+        params['comment'] = comment
+        params['text'] = text
+        return self.post('upload', **params)
 
     def parse(self, text=None, title=None, page=None, prop=None,
               redirects=False, mobileformat=False):
@@ -845,8 +962,7 @@ class Site(object):
                                                    toponly='1' if toponly else None))
         return listing.List(self, 'recentchanges', 'rc', limit=limit, **kwargs)
 
-    def revisions(self, revids, prop='ids|timestamp|flags|comment|user',
-                  expandtemplates=False, diffto='prev'):
+    def revisions(self, revids, prop='ids|timestamp|flags|comment|user'):
         """Get data about a list of revisions.
 
         See also the `Page.revisions()` method.
@@ -861,10 +977,6 @@ class Site(object):
         Args:
             revids (list): A list of (max 50) revisions.
             prop (str): Which properties to get for each revision.
-            expandtemplates (bool): Expand templates in `rvprop=content` output.
-            diffto (str): Revision ID to diff each revision to. Use "prev",
-                          "next" and "cur" for the previous, next and current
-                          revision respectively.
 
         Returns:
             A list of revisions
@@ -874,10 +986,6 @@ class Site(object):
             'rvprop': prop,
             'revids': '|'.join(map(text_type, revids))
         }
-        if expandtemplates:
-            kwargs['rvexpandtemplates'] = '1'
-        if diffto:
-            kwargs['rvdiffto'] = diffto
 
         revisions = []
         pages = self.get('query', **kwargs).get('query', {}).get('pages', {}).values()
@@ -921,7 +1029,7 @@ class Site(object):
         return listing.List(self, 'search', 'sr', limit=limit, **kwargs)
 
     def usercontributions(self, user, start=None, end=None, dir='older', namespace=None,
-                          prop=None, show=None, limit=None):
+                          prop=None, show=None, limit=None, uselang=None):
         """
         List the contributions made by a given user to the wiki, à la Special:Contributions.
 
@@ -930,7 +1038,8 @@ class Site(object):
         kwargs = dict(listing.List.generate_kwargs('uc', user=user, start=start, end=end,
                                                    dir=dir, namespace=namespace,
                                                    prop=prop, show=show))
-        return listing.List(self, 'usercontribs', 'uc', limit=limit, **kwargs)
+        return listing.List(self, 'usercontribs', 'uc', limit=limit, uselang=uselang,
+                            **kwargs)
 
     def users(self, users, prop='blockinfo|groups|editcount'):
         """
@@ -1005,6 +1114,13 @@ class Site(object):
                 query=query, offset=offset), http_method='GET', **kwargs)
             self.handle_api_result(results)  # raises APIError on error
             offset = results.get('query-continue-offset')
-            answers = results['query'].get('results') or {}
-            for key, value in answers.items():
-                yield {key: value}
+            answers = results['query'].get('results', [])
+
+            if isinstance(answers, dict):
+                # In older versions of Semantic MediaWiki (at least until 2.3.0)
+                # a list was returned. In newer versions an object is returned
+                # with the page title as key.
+                answers = [answer for answer in answers.values()]
+
+            for answer in answers:
+                yield answer
