@@ -1,7 +1,6 @@
 import six
 from six import text_type
 import time
-import warnings
 from mwclient.util import parse_timestamp
 import mwclient.listing
 import mwclient.errors
@@ -123,14 +122,7 @@ class Page(object):
     def get_token(self, type, force=False):
         return self.site.get_token(type, force, title=self.name)
 
-    def edit(self, *args, **kwargs):
-        """Deprecated. Use page.text() instead"""
-        warnings.warn("page.edit() was deprecated in mwclient 0.7.0 "
-                      "and will be removed in 0.9.0, please use page.text() instead.",
-                      category=DeprecationWarning, stacklevel=2)
-        return self.text(*args, **kwargs)
-
-    def text(self, section=None, expandtemplates=False, cache=True):
+    def text(self, section=None, expandtemplates=False, cache=True, slot='main'):
         """Get the current wikitext of the page, or of a specific section.
 
         If the page does not exist, an empty string is returned. By
@@ -156,17 +148,23 @@ class Page(object):
         if cache and key in self._textcache:
             return self._textcache[key]
 
-        revs = self.revisions(prop='content|timestamp', limit=1, section=section,
-                              expandtemplates=expandtemplates)
+        revs = self.revisions(prop='content|timestamp', limit=1, section=section, slots=slot)
         try:
             rev = next(revs)
-            text = rev['*']
+            if 'slots' in rev:
+                text = rev['slots'][slot]['*']
+            else:
+                text = rev['*']
             self.last_rev_time = rev['timestamp']
         except StopIteration:
             text = u''
             self.last_rev_time = None
         if not expandtemplates:
             self.edit_time = time.gmtime()
+        else:
+            # The 'rvexpandtemplates' option was removed in MediaWiki 1.32, so we have to make
+            # an extra API call. See: https://github.com/mwclient/mwclient/issues/214
+            text = self.site.expandtemplates(text)
 
         if cache:
             self._textcache[key] = text
@@ -228,6 +226,11 @@ class Page(object):
         if 'newtimestamp' in result['edit'].keys():
             self.last_rev_time = parse_timestamp(result['edit'].get('newtimestamp'))
 
+        # Workaround for https://phabricator.wikimedia.org/T211233
+        for cookie in self.site.connection.cookies:
+            if 'PostEditRevision' in cookie.name:
+                self.site.connection.cookies.clear(cookie.domain, cookie.path, cookie.name)
+
         # clear the page text cache
         self._textcache = {}
         return result['edit']
@@ -326,18 +329,33 @@ class Page(object):
             **kwargs
         )
 
-    def categories(self, generator=True):
+    def categories(self, generator=True, show=None):
         """List categories used on the current page.
 
         API doc: https://www.mediawiki.org/wiki/API:Categories
 
+        Args:
+            generator (bool): Return generator (Default: True)
+            show (str): Set to 'hidden' to only return hidden categories
+                or '!hidden' to only return non-hidden ones.
+
+        Returns:
+            mwclient.listings.PagePropertyGenerator
         """
+        prefix = mwclient.listing.List.get_prefix('cl', generator)
+        kwargs = dict(mwclient.listing.List.generate_kwargs(
+            prefix, show=show
+        ))
+
         if generator:
-            return mwclient.listing.PagePropertyGenerator(self, 'categories', 'cl')
+            return mwclient.listing.PagePropertyGenerator(
+                self, 'categories', 'cl', **kwargs
+            )
         else:
             # TODO: return sortkey if wanted
-            return mwclient.listing.PageProperty(self, 'categories', 'cl',
-                                                 return_values='title')
+            return mwclient.listing.PageProperty(
+                self, 'categories', 'cl', return_values='title', **kwargs
+            )
 
     def embeddedin(self, namespace=None, filterredir='all', limit=None, generator=True):
         """List pages that transclude the current page.
@@ -349,7 +367,7 @@ class Page(object):
             filterredir (str): How to filter redirects, either 'all' (default),
                 'redirects' or 'nonredirects'.
             limit (int): Maximum amount of pages to return per request
-            generator (bool): Use generator
+            generator (bool): Return generator (Default: True)
 
         Returns:
             mwclient.listings.List: Page iterator
@@ -424,7 +442,7 @@ class Page(object):
                   dir='older', user=None, excludeuser=None, limit=50,
                   prop='ids|timestamp|flags|comment|user',
                   expandtemplates=False, section=None,
-                  diffto=None):
+                  diffto=None, slots=None, uselang=None):
         """List revisions of the current page.
 
         API doc: https://www.mediawiki.org/wiki/API:Revisions
@@ -445,15 +463,26 @@ class Page(object):
             diffto (str): Revision ID to diff each revision to. Use "prev",
                           "next" and "cur" for the previous, next and current
                           revision respectively.
+            slots (str): The content slot (Mediawiki >= 1.32) to retrieve
+                content from.
+            uselang (str): Language to use for parsed edit comments and other
+                           localized messages.
 
         Returns:
             mwclient.listings.List: Revision iterator
         """
         kwargs = dict(mwclient.listing.List.generate_kwargs('rv', startid=startid, endid=endid,
                                                             start=start, end=end, user=user,
-                                                            excludeuser=excludeuser, diffto=diffto))
+                                                            excludeuser=excludeuser, diffto=diffto,
+                                                            slots=slots))
+
+        if self.site.version[:2] < (1, 32) and 'rvslots' in kwargs:
+            # https://github.com/mwclient/mwclient/issues/199
+            del kwargs['rvslots']
+
         kwargs['rvdir'] = dir
         kwargs['rvprop'] = prop
+        kwargs['uselang'] = uselang
         if expandtemplates:
             kwargs['rvexpandtemplates'] = '1'
         if section is not None:
